@@ -1,13 +1,13 @@
 package ttlcache_test
 
 import (
+	"go.uber.org/goleak"
 	"math/rand"
+	"strconv"
 	"sync/atomic"
 	"testing"
 	"time"
 
-	"go.uber.org/goleak"
-
 	"fmt"
 	"sync"
 
@@ -367,9 +367,11 @@ func TestCache_SetExpirationCallback(t *testing.T) {
 	cache := NewCache()
 	defer cache.Close()
 
+	ch := make(chan struct{}, 1024)
 	cache.SetTTL(time.Second * 1)
 	cache.SetExpirationCallback(func(key string, value interface{}) {
 		t.Logf("This key(%s) has expired\n", key)
+		ch <- struct{}{}
 	})
 	for i := 0; i < 1024; i++ {
 		cache.Set(fmt.Sprintf("item_%d", i), A{})
@@ -380,6 +382,12 @@ func TestCache_SetExpirationCallback(t *testing.T) {
 	if cache.Count() > 100 {
 		t.Fatal("Cache should empty entries >1 second old")
 	}
+
+	expired := 0
+	for expired != 1024 {
+		<-ch
+		expired++
+	}
 }
 
 // test github issue #4
@@ -661,3 +669,23 @@ func TestCache_Purge(t *testing.T) {
 	}
 
 }
+
+func TestCache_Limit(t *testing.T) {
+	t.Parallel()
+
+	cache := NewCache()
+	defer cache.Close()
+
+	cache.SetTTL(time.Duration(100 * time.Second))
+	cache.SetCacheSizeLimit(10)
+
+	for i := 0; i < 100; i++ {
+		cache.Set("key"+strconv.FormatInt(int64(i), 10), "value")
+	}
+	assert.Equal(t, 10, cache.Count(), "Cache should equal to limit")
+	for i := 90; i < 100; i++ {
+		key := "key" + strconv.FormatInt(int64(i), 10)
+		val, _ := cache.Get(key)
+		assert.Equal(t, "value", val, "Cache should be set [key90, key99]")
+	}
+}
